<?php

/**
 * @file
 * Contains \Drupal\readme\ReadmeManager.
 */

namespace Drupal\readme;

use Michelf\Markdown;

// If Markdown is not autoloaded the attempt to include /libraries/markdown.
if (!class_exists('Michelf\Markdown')) {
  include_once \Drupal::root() . '/libraries/markdown/Michelf/Markdown.inc.php';
}

/**
 * Manages module README files.
 */
class ReadmeManager implements ReadmeManagerInterface {

  /**
   * {@inheritdoc}
   */
  public function markdownInstalled() {
    return class_exists('Michelf\Markdown');
  }

  /**
   * {@inheritdoc}
   */
  public function exists($module_name) {
    return ($this->getPath($module_name)) ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getPath($module_name) {
    $module_path = drupal_get_path('module', $module_name);
    if (file_exists("$module_path/README.txt")) {
      return "$module_path/README.txt";
    }
    elseif (file_exists("$module_path/README.md")) {
      return "$module_path/README.md";
    }
    else {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isMarkDown($module_name) {
    $readme_path = $this->getPath($module_name);
    if (!$readme_path) {
      return FALSE;
    }
    return preg_match('/\.md$/', $readme_path) ? TRUE : FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function getHtml($module_name) {
    $readme_path = $this->getPath($module_name);
    if (!$readme_path) {
      return FALSE;
    }

    $content = file_get_contents($readme_path);
    if ($this->isMarkDown($module_name) && $this->markdownInstalled()) {
      $html = Markdown::defaultTransform($content);
      return $this->tidy($html);
    }
    else {
      $html = htmlentities($content);
      // Hyper link URLs using (hidden) _filter_url function.
      $filter = (object) [
        'settings' => [
          'filter_url_length' => NULL,
        ],
      ];
      $html = _filter_url($html, $filter);
      return '<pre>' . $html . '</pre>';
    }
  }

  /**
   * Tidy an HTML string.
   *
   * @param string $html
   *   HTML string to be tidied.
   *
   * @return string
   *   A tidied HTML string.
   */
  protected function tidy($html) {
    if (!class_exists('\tidy')) {
      return $html;
    }

    // Configuration.
    // - http://us3.php.net/manual/en/book.tidy.php
    // - http://tidy.sourceforge.net/docs/quickref.html#wrap
    $config = ['show-body-only' => TRUE, 'wrap' => '0'];

    $tidy = new \tidy();
    $tidy->parseString($html, $config, 'utf8');
    $tidy->cleanRepair();
    $html = tidy_get_output($tidy);

    // Remove <code> tag nested within <pre> tag.
    $html = str_replace('<pre><code>', '<pre>', $html);
    $html = str_replace('</code></pre>', '</pre>', $html);

    return $html;
  }

}
